//! @brief maintain a list of on-the-fly objects
//! @copyright (c) since 2024, see LICENSE.txt
#ifndef OTF_LIST_H
#define OTF_LIST_H

#include "otf_stats.h"

#include <memory>
#include <string>
#include <vector>

//! maintain a list of OTFStats objects for easier reporting
class OTFList {
public:
    //! create an OTFS object with the given tag name
    //! \param[in] tag  the name of the OTFS object
    void create(std::string tag);

    //! get a reference to an OTFS object with the given tag name.
    //! Throws an exception if not found.
    //! \param[in] tag  the name of the OTFS object to get
    //! \return pointer to the OTFS object if found
    auto get(const std::string& tag) -> std::shared_ptr<OTFStats>;

    //! run report on all OTFS objects
    //! \param writeln  callback function to print the given line
    void report(OTFStats::writeln_func writeln) const;

private:
    //! list of OFTStats objects to report
    std::vector<std::shared_ptr<OTFStats>> m_objs;
};

#endif // OTF_LIST_H
